@@ -37,7 +37,7 @@ module OmniAuth
       def request_phase
         form = OmniAuth::Form.new(:title => 'User Info', :url => callback_path)
         options.fields.each do |field|
-          form.text_field field.to_s.capitalize.gsub('_', ' '), field.to_s
+          form.text_field field.to_s.capitalize.tr('_', ' '), field.to_s
         end
         form.button 'Sign In'
         form.to_response
